@@ -18,6 +18,7 @@ from .common import (
     DocstringReturns,
     DocstringStyle,
     ParseError,
+    RenderingStyle,
 )
 
 
@@ -41,8 +42,8 @@ def _build_meta(args: T.List[str], desc: str) -> DocstringMeta:
                 "Expected one or two arguments for a {} keyword.".format(key)
             )
 
-        m = re.match(r".*defaults to (.+)", desc, flags=re.DOTALL)
-        default = m.group(1).rstrip(".") if m else None
+        match = re.match(r".*defaults to (.+)", desc, flags=re.DOTALL)
+        default = match.group(1).rstrip(".") if match else None
 
         return DocstringParam(
             args=args,
@@ -72,20 +73,15 @@ def _build_meta(args: T.List[str], desc: str) -> DocstringMeta:
 
     if key in DEPRECATION_KEYWORDS:
         match = re.search(
-            "^(?P<version>v?((?:\d+)(?:\.[0-9a-z\.]+))) (?P<desc>.+)",
+            r"^(?P<version>v?((?:\d+)(?:\.[0-9a-z\.]+))) (?P<desc>.+)",
             desc,
             flags=re.I,
         )
-        if match:
-            return DocstringDeprecated(
-                args=args,
-                version=match.group("version"),
-                description=match.group("desc"),
-            )
-        else:
-            return DocstringDeprecated(
-                args=args, version=None, description=desc
-            )
+        return DocstringDeprecated(
+            args=args,
+            version=match.group("version") if match else None,
+            description=match.group("desc") if match else desc,
+        )
 
     if key in RAISES_KEYWORDS:
         if len(args) == 2:
@@ -108,7 +104,7 @@ def parse(text: str) -> Docstring:
 
     :returns: parsed docstring
     """
-    ret = Docstring(style=DocstringStyle.rest)
+    ret = Docstring(style=DocstringStyle.REST)
     if not text:
         return ret
 
@@ -137,10 +133,10 @@ def parse(text: str) -> Docstring:
             continue
         try:
             args_chunk, desc_chunk = chunk.lstrip(":").split(":", 1)
-        except ValueError:
+        except ValueError as ex:
             raise ParseError(
                 'Error parsing meta information near "{}".'.format(chunk)
-            )
+            ) from ex
         args = args_chunk.split()
         desc = desc_chunk.strip()
         if "\n" in desc:
@@ -150,3 +146,71 @@ def parse(text: str) -> Docstring:
         ret.meta.append(_build_meta(args, desc))
 
     return ret
+
+
+def compose(
+    docstring: Docstring,
+    rendering_style: RenderingStyle = RenderingStyle.COMPACT,
+    indent: str = "    ",
+) -> str:
+    """Render a parsed docstring into docstring text.
+
+    :param docstring: parsed docstring representation
+    :param rendering_style: the style to render docstrings
+    :param indent: the characters used as indentation in the docstring string
+    :returns: docstring text
+    """
+
+    def process_desc(desc: T.Optional[str]) -> str:
+        if not desc:
+            return ""
+
+        if rendering_style == RenderingStyle.CLEAN:
+            (first, *rest) = desc.splitlines()
+            return "\n".join([" " + first] + [indent + line for line in rest])
+
+        if rendering_style == RenderingStyle.EXPANDED:
+            (first, *rest) = desc.splitlines()
+            return "\n".join(
+                ["\n" + indent + first] + [indent + line for line in rest]
+            )
+
+        return " " + desc
+
+    parts: T.List[str] = []
+    if docstring.short_description:
+        parts.append(docstring.short_description)
+    if docstring.blank_after_short_description:
+        parts.append("")
+    if docstring.long_description:
+        parts.append(docstring.long_description)
+    if docstring.blank_after_long_description:
+        parts.append("")
+
+    for meta in docstring.meta:
+        if isinstance(meta, DocstringParam):
+            if meta.type_name:
+                type_text = (
+                    f" {meta.type_name}? "
+                    if meta.is_optional
+                    else f" {meta.type_name} "
+                )
+            else:
+                type_text = " "
+            text = f":param{type_text}{meta.arg_name}:"
+            text += process_desc(meta.description)
+            parts.append(text)
+        elif isinstance(meta, DocstringReturns):
+            type_text = f" {meta.type_name} " if meta.type_name else ""
+            key = "yields" if meta.is_generator else "returns"
+            text = f":{key}{type_text}:"
+            text += process_desc(meta.description)
+            parts.append(text)
+        elif isinstance(meta, DocstringRaises):
+            type_text = f" {meta.type_name} " if meta.type_name else ""
+            text = f"@raises{type_text}:" + process_desc(meta.description)
+            parts.append(text)
+        else:
+            text = f'@{" ".join(meta.args)}:' + process_desc(meta.description)
+            parts.append(text)
+    return "\n".join(parts)
