@@ -1,13 +1,23 @@
+"""Tests for Google-style docstring routines."""
 import typing as T
 
 import pytest
-from docstring_parser.common import ParseError
-from docstring_parser.google import GoogleParser, Section, SectionType, parse
+from docstring_parser.common import ParseError, RenderingStyle
+from docstring_parser.google import (
+    GoogleParser,
+    Section,
+    SectionType,
+    compose,
+    parse,
+)
 
 
-def test_google_parser():
+def test_google_parser_unknown_section() -> None:
+    """Test parsing an unknown section with default GoogleParser
+    configuration.
+    """
     parser = GoogleParser()
-    docstring = parse(
+    docstring = parser.parse(
         """
         Unknown:
             spam: a
@@ -17,6 +27,11 @@ def test_google_parser():
     assert docstring.long_description == "spam: a"
     assert len(docstring.meta) == 0
 
+
+def test_google_parser_custom_sections() -> None:
+    """Test parsing an unknown section with custom GoogleParser
+    configuration.
+    """
     parser = GoogleParser(
         [
             Section("DESCRIPTION", "desc", SectionType.SINGULAR),
@@ -61,6 +76,12 @@ def test_google_parser():
     assert docstring.meta[5].args == ["examples"]
     assert docstring.meta[5].description == "Many examples\nMore examples"
 
+
+def test_google_parser_custom_sections_after() -> None:
+    """Test parsing an unknown section with custom GoogleParser configuration
+    that was set at a runtime.
+    """
+    parser = GoogleParser(title_colon=False)
     parser.add_section(Section("Note", "note", SectionType.SINGULAR))
     docstring = parser.parse(
         """
@@ -107,6 +128,7 @@ def test_google_parser():
     ],
 )
 def test_short_description(source: str, expected: str) -> None:
+    """Test parsing short description."""
     docstring = parse(source)
     assert docstring.short_description == expected
     assert docstring.long_description is None
@@ -182,6 +204,7 @@ def test_long_description(
     expected_long_desc: str,
     expected_blank: bool,
 ) -> None:
+    """Test parsing long description."""
     docstring = parse(source)
     assert docstring.short_description == expected_short_desc
     assert docstring.long_description == expected_long_desc
@@ -277,6 +300,7 @@ def test_meta_newlines(
     expected_blank_short_desc: bool,
     expected_blank_long_desc: bool,
 ) -> None:
+    """Test parsing newlines around description sections."""
     docstring = parse(source)
     assert docstring.short_description == expected_short_desc
     assert docstring.long_description == expected_long_desc
@@ -286,6 +310,7 @@ def test_meta_newlines(
 
 
 def test_meta_with_multiline_description() -> None:
+    """Test parsing multiline meta documentation."""
     docstring = parse(
         """
         Short description
@@ -305,21 +330,22 @@ def test_meta_with_multiline_description() -> None:
 
 
 def test_default_args():
+    """Test parsing default arguments."""
     docstring = parse(
         """A sample function
 
-    A function the demonstrates docstrings
+A function the demonstrates docstrings
 
-    Args:
-        arg1 (int): The firsty arg
-        arg2 (str): The second arg
-        arg3 (float, optional): The third arg. Defaults to 1.0.
-        arg4 (Optional[Dict[str, Any]], optional): The fourth arg. Defaults to None.
-        arg5 (str, optional): The fifth arg. Defaults to DEFAULT_ARG5.
+Args:
+    arg1 (int): The firsty arg
+    arg2 (str): The second arg
+    arg3 (float, optional): The third arg. Defaults to 1.0.
+    arg4 (Optional[Dict[str, Any]], optional): The last arg. Defaults to None.
+    arg5 (str, optional): The fifth arg. Defaults to DEFAULT_ARG5.
 
-    Returns:
-        Mapping[str, Any]: The args packed in a mapping
-    """
+Returns:
+    Mapping[str, Any]: The args packed in a mapping
+"""
     )
     assert docstring is not None
     assert len(docstring.params) == 5
@@ -329,10 +355,11 @@ def test_default_args():
     assert arg4.is_optional
     assert arg4.type_name == "Optional[Dict[str, Any]]"
     assert arg4.default == "None"
-    assert arg4.description == "The fourth arg. Defaults to None."
+    assert arg4.description == "The last arg. Defaults to None."
 
 
 def test_multiple_meta() -> None:
+    """Test parsing multiple meta."""
     docstring = parse(
         """
         Short description
@@ -362,6 +389,7 @@ def test_multiple_meta() -> None:
 
 
 def test_params() -> None:
+    """Test parsing params."""
     docstring = parse("Short description")
     assert len(docstring.params) == 0
 
@@ -408,7 +436,7 @@ def test_params() -> None:
     assert docstring.params[0].arg_name == "name"
     assert docstring.params[0].type_name is None
     assert docstring.params[0].description == (
-        "description 1\n" "with multi-line text"
+        "description 1\nwith multi-line text"
     )
     assert docstring.params[1].arg_name == "priority"
     assert docstring.params[1].type_name == "int"
@@ -416,6 +444,7 @@ def test_params() -> None:
 
 
 def test_attributes() -> None:
+    """Test parsing attributes."""
     docstring = parse("Short description")
     assert len(docstring.params) == 0
 
@@ -462,7 +491,7 @@ def test_attributes() -> None:
     assert docstring.params[0].arg_name == "name"
     assert docstring.params[0].type_name is None
     assert docstring.params[0].description == (
-        "description 1\n" "with multi-line text"
+        "description 1\nwith multi-line text"
     )
     assert docstring.params[1].arg_name == "priority"
     assert docstring.params[1].type_name == "int"
@@ -470,6 +499,7 @@ def test_attributes() -> None:
 
 
 def test_returns() -> None:
+    """Test parsing returns."""
     docstring = parse(
         """
         Short description
@@ -561,7 +591,7 @@ def test_returns() -> None:
     assert docstring.returns is not None
     assert docstring.returns.type_name == "int"
     assert docstring.returns.description == (
-        "description\n" "with much text\n\n" "even some spacing"
+        "description\nwith much text\n\neven some spacing"
     )
     assert docstring.many_returns is not None
     assert len(docstring.many_returns) == 1
@@ -569,6 +599,7 @@ def test_returns() -> None:
 
 
 def test_raises() -> None:
+    """Test parsing raises."""
     docstring = parse(
         """
         Short description
@@ -589,6 +620,7 @@ def test_raises() -> None:
 
 
 def test_examples() -> None:
+    """Test parsing examples."""
     docstring = parse(
         """
         Short description
@@ -606,6 +638,7 @@ def test_examples() -> None:
 
 
 def test_broken_meta() -> None:
+    """Test parsing broken meta."""
     with pytest.raises(ParseError):
         parse("Args:")
 
@@ -614,6 +647,7 @@ def test_broken_meta() -> None:
 
 
 def test_unknown_meta() -> None:
+    """Test parsing unknown meta."""
     docstring = parse(
         """Short desc
 
@@ -639,8 +673,9 @@ def test_unknown_meta() -> None:
 
 
 def test_broken_arguments() -> None:
+    """Test parsing broken arguments."""
     with pytest.raises(ParseError):
-        docstring = parse(
+        parse(
             """This is a test
 
             Args:
@@ -650,6 +685,7 @@ def test_broken_arguments() -> None:
 
 
 def test_empty_example() -> None:
+    """Test parsing empty examples section."""
     docstring = parse(
         """Short description
 
@@ -661,5 +697,281 @@ def test_empty_example() -> None:
     )
 
     assert len(docstring.meta) == 2
-    assert docstring.meta[0].args == ['examples']
+    assert docstring.meta[0].args == ["examples"]
     assert docstring.meta[0].description == ""
+
+
+@pytest.mark.parametrize(
+    "source, expected",
+    [
+        ("", ""),
+        ("\n", ""),
+        ("Short description", "Short description"),
+        ("\nShort description\n", "Short description"),
+        ("\n   Short description\n", "Short description"),
+        (
+            "Short description\n\nLong description",
+            "Short description\n\nLong description",
+        ),
+        (
+            """
+            Short description
+
+            Long description
+            """,
+            "Short description\n\nLong description",
+        ),
+        (
+            """
+            Short description
+
+            Long description
+            Second line
+            """,
+            "Short description\n\nLong description\nSecond line",
+        ),
+        (
+            "Short description\nLong description",
+            "Short description\nLong description",
+        ),
+        (
+            """
+            Short description
+            Long description
+            """,
+            "Short description\nLong description",
+        ),
+        (
+            "\nShort description\nLong description\n",
+            "Short description\nLong description",
+        ),
+        (
+            """
+            Short description
+            Long description
+            Second line
+            """,
+            "Short description\nLong description\nSecond line",
+        ),
+        (
+            """
+            Short description
+            Meta:
+                asd
+            """,
+            "Short description\nMeta:\n    asd",
+        ),
+        (
+            """
+            Short description
+            Long description
+            Meta:
+                asd
+            """,
+            "Short description\nLong description\nMeta:\n    asd",
+        ),
+        (
+            """
+            Short description
+            First line
+                Second line
+            Meta:
+                asd
+            """,
+            "Short description\n"
+            "First line\n"
+            "    Second line\n"
+            "Meta:\n"
+            "    asd",
+        ),
+        (
+            """
+            Short description
+
+            First line
+                Second line
+            Meta:
+                asd
+            """,
+            "Short description\n"
+            "\n"
+            "First line\n"
+            "    Second line\n"
+            "Meta:\n"
+            "    asd",
+        ),
+        (
+            """
+            Short description
+
+            First line
+                Second line
+
+            Meta:
+                asd
+            """,
+            "Short description\n"
+            "\n"
+            "First line\n"
+            "    Second line\n"
+            "\n"
+            "Meta:\n"
+            "    asd",
+        ),
+        (
+            """
+            Short description
+
+            Meta:
+                asd
+                    1
+                        2
+                    3
+            """,
+            "Short description\n"
+            "\n"
+            "Meta:\n"
+            "    asd\n"
+            "        1\n"
+            "            2\n"
+            "        3",
+        ),
+        (
+            """
+            Short description
+
+            Meta1:
+                asd
+                1
+                    2
+                3
+            Meta2:
+                herp
+            Meta3:
+                derp
+            """,
+            "Short description\n"
+            "\n"
+            "Meta1:\n"
+            "    asd\n"
+            "    1\n"
+            "        2\n"
+            "    3\n"
+            "Meta2:\n"
+            "    herp\n"
+            "Meta3:\n"
+            "    derp",
+        ),
+        (
+            """
+            Short description
+
+            Args:
+                name: description 1
+                priority (int): description 2
+                sender (str, optional): description 3
+                message (str, optional): description 4, defaults to 'hello'
+                multiline (str?):
+                    long description 5,
+                        defaults to 'bye'
+            """,
+            "Short description\n"
+            "\n"
+            "Args:\n"
+            "    name: description 1\n"
+            "    priority (int): description 2\n"
+            "    sender (str?): description 3\n"
+            "    message (str?): description 4, defaults to 'hello'\n"
+            "    multiline (str?): long description 5,\n"
+            "        defaults to 'bye'",
+        ),
+        (
+            """
+            Short description
+            Raises:
+                ValueError: description
+            """,
+            "Short description\nRaises:\n    ValueError: description",
+        ),
+    ],
+)
+def test_compose(source: str, expected: str) -> None:
+    """Test compose in default mode."""
+    assert compose(parse(source)) == expected
+
+
+@pytest.mark.parametrize(
+    "source, expected",
+    [
+        (
+            """
+            Short description
+
+            Args:
+                name: description 1
+                priority (int): description 2
+                sender (str, optional): description 3
+                message (str, optional): description 4, defaults to 'hello'
+                multiline (str?):
+                    long description 5,
+                        defaults to 'bye'
+            """,
+            "Short description\n"
+            "\n"
+            "Args:\n"
+            "    name: description 1\n"
+            "    priority (int): description 2\n"
+            "    sender (str, optional): description 3\n"
+            "    message (str, optional): description 4, defaults to 'hello'\n"
+            "    multiline (str, optional): long description 5,\n"
+            "        defaults to 'bye'",
+        ),
+    ],
+)
+def test_compose_clean(source: str, expected: str) -> None:
+    """Test compose in clean mode."""
+    assert (
+        compose(parse(source), rendering_style=RenderingStyle.CLEAN)
+        == expected
+    )
+
+
+@pytest.mark.parametrize(
+    "source, expected",
+    [
+        (
+            """
+            Short description
+
+            Args:
+                name: description 1
+                priority (int): description 2
+                sender (str, optional): description 3
+                message (str, optional): description 4, defaults to 'hello'
+                multiline (str?):
+                    long description 5,
+                        defaults to 'bye'
+            """,
+            "Short description\n"
+            "\n"
+            "Args:\n"
+            "    name:\n"
+            "        description 1\n"
+            "    priority (int):\n"
+            "        description 2\n"
+            "    sender (str, optional):\n"
+            "        description 3\n"
+            "    message (str, optional):\n"
+            "        description 4, defaults to 'hello'\n"
+            "    multiline (str, optional):\n"
+            "        long description 5,\n"
+            "        defaults to 'bye'",
+        ),
+    ],
+)
+def test_compose_expanded(source: str, expected: str) -> None:
+    """Test compose in expanded mode."""
+    assert (
+        compose(parse(source), rendering_style=RenderingStyle.EXPANDED)
+        == expected
+    )
